# Copyright (c) 2018 NTT DATA
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.conf import settings
from django.urls import reverse
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import tables
from horizon import tabs
from horizon.utils import memoized

from masakaridashboard.api import api
from masakaridashboard.notifications import tables as notification_tab
from masakaridashboard.notifications import tabs as not_tab


class IndexView(tables.DataTableView):
    table_class = notification_tab.NotificationsTable
    template_name = 'masakaridashboard/notifications/index.html'
    page_title = _("Notifications")

    _more = False
    _prev = False

    def needs_filter_first(self, table):
        return self._needs_filter_first

    def has_more_data(self, table):
        return self._more

    def has_prev_data(self, table):
        return self._prev

    def get_data(self):
        notification_list = []

        marker = self.request.GET.get(
            notification_tab.NotificationsTable._meta.pagination_param,
            None
        )
        if marker is not None:
            try:
                notification = api.get_notification(self.request, marker)
                marker = notification.id
            except Exception:
                msg = _('Unable to get notification "%s".') % marker
                redirect = reverse(
                    'horizon:masakaridashboard:notifications:index')
                exceptions.handle(self.request, msg, redirect=redirect)

        filters = self.get_filters()
        self._needs_filter_first = True

        filter_first = getattr(settings, 'FILTER_DATA_FIRST', {})
        if filter_first.get('masakaridashboard.notifications', False) and len(
                filters) == 0:
            self._needs_filter_first = True
            self._more = False
            return notification_list
        try:
            notification_list, self._more, self._prev = api.notification_list(
                self.request, filters=filters, marker=marker, paginate=True)
        except Exception:
            self._prev = False
            self._more = False
            msg = _('Unable to retrieve notification list.')
            exceptions.handle(self.request, msg)
        return notification_list


class DetailView(tabs.TabbedTableView):
    tab_group_class = not_tab.NotificationDetailTabs
    template_name = 'horizon/common/_detail.html'
    page_title = "{{ notification.notification_uuid }}"

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        notification = self.get_data()
        table = notification_tab.NotificationsTable(self.request)
        context["notification"] = notification
        context["url"] = self.get_redirect_url()
        context["actions"] = table.render_row_actions(notification)
        return context

    @memoized.memoized_method
    def get_data(self):
        try:
            notification_id = self.kwargs['notification_id']
            notification = api.get_notification(self.request, notification_id)
        except Exception:
            msg = _('Unable to get notification "%s".') % notification_id
            redirect = reverse('horizon:masakaridashboard:notifications:index')
            exceptions.handle(self.request, msg, redirect=redirect)

        return notification

    def get_redirect_url(self):
        return reverse('horizon:masakaridashboard:notifications:index')

    def get_tabs(self, request, *args, **kwargs):
        notification = self.get_data()
        return self.tab_group_class(
            request, notification=notification, **kwargs)



# Copyright (c) 2018 NTT DATA
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.urls import reverse
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import messages

from masakaridashboard.api import api


class UpdateHostForm(forms.SelfHandlingForm):

    uuid = forms.CharField(widget=forms.HiddenInput())
    failover_segment_id = forms.CharField(widget=forms.HiddenInput())
    name = forms.CharField(
        label=_('Host Name'),
        widget=forms.TextInput(attrs={'readonly': 'readonly'}))
    reserved = forms.ChoiceField(
        label=_('Reserved'),
        choices=[('False', 'False'),
                 ('True', 'True')],
        widget=forms.Select(
            attrs={'class': 'switchable',
                   'data-slug': 'available host'}),
        required=False)
    type = forms.CharField(
        label=_('Type'),
        widget=forms.TextInput(attrs={'maxlength': 255}))
    control_attributes = forms.CharField(
        label=_('Control Attribute'),
        widget=forms.TextInput())
    on_maintenance = forms.ChoiceField(
        label=_('On Maintenance'),
        choices=[('False', 'False'),
                 ('True', 'True')],
        widget=forms.Select(
            attrs={'class': 'switchable',
                   'data-slug': 'available host'}),
        required=False
    )

    def handle(self, request, data):
        try:
            attrs = {'name': data['name'],
                     'reserved': data['reserved'],
                     'type': data['type'],
                     'control_attributes': data['control_attributes'],
                     'on_maintenance': data['on_maintenance']}
            api.update_host(request, data['uuid'],
                            data["failover_segment_id"], attrs)
            msg = _('Successfully updated host.')
            messages.success(request, msg)
        except Exception:
            msg = _('Failed to update host.')
            redirect = reverse('horizon:masakaridashboard:hosts:index')
            exceptions.handle(request, msg, redirect=redirect)
        return True



# Copyright (c) 2018 NTT DATA
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.utils.translation import ugettext_lazy as _

from horizon import tables
from horizon.utils import filters


NOTIFICATION_FILTER_CHOICES = (
    ('source_host_uuid', _("Source Host UUID ="), True),
    ('type', _("Type ="), True),
    ('status', _("Status ="), True),
    ('generated_since', _("Generated Since ="), True),
)


class NotificationFilterAction(tables.FilterAction):
    filter_type = "server"
    filter_choices = NOTIFICATION_FILTER_CHOICES


class ProgressDetailsItem(object):
    def __init__(self, id, action, timestamp, message):

        self.id = id
        self.action = action
        self.timestamp = timestamp
        self.message = message


class NotificationsTable(tables.DataTable):

    source_host_uuid = tables.Column(
        'source_host_uuid', verbose_name=_("Host"))
    notification_uuid = tables.Column(
        'notification_uuid', verbose_name=_("UUID"),
        link="horizon:masakaridashboard:notifications:detail")
    type = tables.Column('type', verbose_name=_("Type"))
    status = tables.Column('status', verbose_name=_("Status"))
    payload = tables.Column(
        'payload', verbose_name=_("Payload"), truncate=40)

    def get_object_id(self, datum):
        return datum.notification_uuid

    class Meta(object):
        name = "notifications"
        verbose_name = _("Notifications")
        table_actions = (NotificationFilterAction,)


class NotificationProgressDetailsTable(tables.DataTable):

    action = tables.Column('action', verbose_name=_('Action'))
    timestamp = tables.Column('timestamp', verbose_name=_('Timestamp'),
                              filters=[filters.parse_isotime])
    message = tables.Column('message', verbose_name=_('Message'))
    id = tables.Column('id', verbose_name=_('ID'), hidden=True)

    class Meta(object):
        name = "notification_progress_details"
        verbose_name = _("NotificationProgressDetails")



# Copyright (C) 2018 NTT DATA
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.urls import reverse
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import messages

from masakaridashboard.api import api


class CreateSegmentForm(forms.SelfHandlingForm):
    name = forms.CharField(
        label=_('Segment Name'),
        widget=forms.TextInput(attrs={'maxlength': 255}),
        help_text=_('The segment name.'))
    recovery_method = forms.ChoiceField(
        label=_('Recovery Method'),
        choices=[('auto', 'auto'),
                 ('auto_priority', 'auto_priority'),
                 ('reserved_host', 'reserved_host'),
                 ('rh_priority', 'rh_priority')],
        widget=forms.Select(
            attrs={'class': 'switchable',
                   'data-slug': 'recovery_method'}),
        required=True,
        help_text=_('Type of recovery if any host in this segment goes down.')
    )
    service_type = forms.CharField(
        label=_('Service Type'),
        help_text=_('The name of service which will be deployed in this'
                    ' segment. As of now user can mention COMPUTE as service'
                    ' type.'),
        widget=forms.TextInput(attrs={
            'readonly': 'readonly', 'value': 'compute'}))
    description = forms.CharField(
        label=_("Description"), widget=forms.Textarea(
            attrs={'rows': 4}), required=False)

    def __init__(self, *args, **kwargs):
        super(CreateSegmentForm, self).__init__(*args, **kwargs)

    def handle(self, request, data):
        try:
            api.segment_create(request, data)
            msg = _('Successfully created segment')
            messages.success(request, msg)
        except Exception as exc:
            if exc.status_code == 409:
                msg = _('Segment with name "%s" already exists') % data["name"]
            else:
                msg = _('Failed to create segment')
            redirect = reverse('horizon:masakaridashboard:segments:index')
            exceptions.handle(request, msg, redirect=redirect)
        return True


class UpdateForm(forms.SelfHandlingForm):
    uuid = forms.CharField(widget=forms.HiddenInput())
    name = forms.CharField(
        label=_('Segment Name'),
        widget=forms.TextInput(attrs={'maxlength': 255}))
    recovery_method = forms.ChoiceField(
        label=_('Recovery Method'),
        choices=[('auto', 'auto'),
                 ('auto_priority', 'auto_priority'),
                 ('reserved_host', 'reserved_host'),
                 ('rh_priority', 'rh_priority')],
        widget=forms.Select(
            attrs={'class': 'switchable',
                   'data-slug': 'recovery_method'}),
        required=False
    )
    description = forms.CharField(
        label=_('Description'),
        widget=forms.Textarea(
            attrs={'width': "100%", 'cols': "80", 'rows': "5", }),
        required=False
    )

    def handle(self, request, data):
        try:
            fields_to_update = {'name': data['name'],
                                'recovery_method': data['recovery_method'],
                                'description': data['description']}
            api.segment_update(request, data['uuid'], fields_to_update)
            msg = _('Successfully updated segment.')
            messages.success(request, msg)
        except Exception:
            msg = _('Failed to update segment.')
            redirect = reverse('horizon:masakaridashboard:segments:index')
            exceptions.handle(request, msg, redirect=redirect)

        return True


class AddHostForm(forms.SelfHandlingForm):

    segment_id = forms.CharField(widget=forms.HiddenInput())
    segment_name = forms.CharField(
        label=_('Segment Name'), widget=forms.TextInput(
            attrs={'readonly': 'readonly'}), required=False)
    name = forms.ChoiceField(label=_('Host Name'),
                             choices=[])
    reserved = forms.ChoiceField(
        label=_('Reserved'),
        choices=[('False', 'False'),
                 ('True', 'True')],
        widget=forms.Select(
            attrs={'class': 'switchable',
                   'data-slug': 'available host'}),
        required=False,
        help_text=_("A boolean indicating whether this host is reserved or"
                    " not. Default value is set to False."))
    type = forms.CharField(
        label=_('Type'),
        widget=forms.TextInput(attrs={'maxlength': 255}),
        help_text=_("Type of host."))
    control_attributes = forms.CharField(
        label=_('Control Attribute'),
        widget=forms.TextInput(),
        help_text=_("Attributes to control host."))
    on_maintenance = forms.ChoiceField(
        label=_('On Maintenance'),
        choices=[('False', 'False'),
                 ('True', 'True')],
        widget=forms.Select(
            attrs={'class': 'switchable',
                   'data-slug': 'available host'}),
        required=False,
        help_text=_("A boolean indicating whether this host is on maintenance"
                    " or not. Default value is set to False."))

    def __init__(self, *args, **kwargs):
        super(AddHostForm, self).__init__(*args, **kwargs)

        # Populate candidate name choices
        available_host_list = kwargs.get('initial', {}).get(
            "available_host_list", [])
        host_candidate_list = []
        # NOTE(pas-ha) available_host_list contains
        # novaclient v2 Service objects
        for service in available_host_list:
            host_candidate_list.append(
                (service.host, '%(name)s (%(id)s)'
                 % {"name": service.host,
                    "id": service.id}))
        if host_candidate_list:
            host_candidate_list.insert(0, ("", _("Select a host")))
        else:
            host_candidate_list.insert(0, ("", _("No host available")))
        self.fields['name'].choices = host_candidate_list

    def handle(self, request, data):
        try:
            api.create_host(request, data)
            msg = _('Host created successfully.')
            messages.success(request, msg)
        except Exception:
            msg = _('Failed to create host.')
            redirect = reverse('horizon:masakaridashboard:segments:index')
            exceptions.handle(request, msg, redirect=redirect)

        return True
